define(
      "datagrid/dgrid/CPBDataGrid", ['cpb', 'menu/dojo/CPBMenu', 'datagrid/dgrid/BaseGrid', 'datagrid/dgrid/RefreshingGridTracker',
       'dijit/Toolbar', 'dijit/registry', 'dijit/form/DropDownButton', 'dijit/DropDownMenu', 'dijit/MenuItem',
       'dgrid/Grid', 'dgrid/selector', 'dgrid/Selection', 'dgrid/extensions/Pagination', 'dgrid/extensions/ColumnResizer',
       'dgrid/extensions/ColumnHider', 'dgrid/extensions/ColumnReorder', 'dgrid/extensions/DijitRegistry',
       'dojo/store/JsonRest', 'dojo/store/Observable', 'dojo/store/Cache', 'dojo/store/Memory',
       'dojo/store/DataStore', 'dojo/data/ItemFileWriteStore', 'dojox/widget/Standby',
       'dojo/_base/declare', 'dojo/_base/array', 'dojo/request/xhr', 'dojo/query', 'dojo/dom-construct', 'dojo/aspect',
       'dojo/dom-attr', 'dojo/has', 'dojo/dom-class', 'dojo/domReady!'],
      function(cpb, CPBMenu, BaseGrid, RefreshingGridTracker,
            Toolbar, registry, DropDownButton, DropDownMenu, MenuItem,
            Grid, selector, Selection, Pagination, ColumnResizer, ColumnHider, ColumnReorder, DijitRegistry,
            JsonRest, Observable, Cache, Memory,
            DataStore, ItemFileWriteStore, Standby,
            declare, array, xhr, query, domConstruct, aspect,
            domAttr, has, domClass) {

var CPBDataGrid = {};

CPBDataGrid.storeMap = {};
CPBDataGrid.gridMap = {};

CPBDataGrid.getGrid = function(gridName)
{
   return CPBDataGrid.gridMap[gridName];
};

CPBDataGrid.clearGridAndReload = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   cpb.consoleLog("CPBDataGrid.clearGridAndReload:: grid, " + grid.cpb_id + " visible?? " + grid.visible);
   if (grid.visible == true)
   {
      grid.refresh(); // TODO: This seems slow, is there a better way?
      //grid.resetToPage(1);
   }
};

CPBDataGrid.getSelectedRowKeys = function(grid, keyColumnName)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   return grid.cpb_getSelectedRowValues(keyColumnName);
};

CPBDataGrid.createMemoryStore = function(gridName, data)
{
   var store = new Memory({ data: data });
   CPBDataGrid.storeMap[gridName] = store;
   return CPBDataGrid.getStore(gridName);
};

CPBDataGrid.createStore = function(gridName, url, useUIContext)
{
   var store = null;
   if (useUIContext)
   {
      store = new JsonRest({ target: url, sortParam: "sortBy" });
      store = Cache(store, Memory());
   }
   else
   {
      var datastore = new ItemFileWriteStore({ url: url, hierarchical: false });
      store = new DataStore({ store: datastore });//Cache(store, Memory())
   }
   cpb.consoleLog('Grid ' + gridName + ' fetches data restfully? - ' + useUIContext);
   CPBDataGrid.storeMap[gridName] = store;
   return CPBDataGrid.getStore(gridName);
};

CPBDataGrid.getStore = function(gridName)
{
   return CPBDataGrid.storeMap[gridName];
};

CPBDataGrid.showMaskForAllGridByStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid){
      CPBDataGrid.showMask(storeGrid);
   });
};

CPBDataGrid.showMask = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.showMask();
};

CPBDataGrid.hideMaskForAllGridByStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid){
      CPBDataGrid.hideMask(storeGrid);
   });
};

CPBDataGrid.hideMask = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.hideMask();
};

isString = function(field)
{
   return (typeof(field) == 'string');
};

/**
 * Refresh a server store, then redraw all grids backed by it.
 */
CPBDataGrid.refreshServerStore = function(storeName)
{
   // show masks on each grid.
   runForAllGridsInStore(storeName, function(storeGrid){
      RefreshingGridTracker.addRefreshingGrid(storeGrid);
      CPBDataGrid.showMask(storeGrid);
   });
   CPBService.clearStoreData(storeName, cpb.getFormParams(),
   {
      async: false, 
      callback: function(isCleared) 
      {
         // refresh all grids associated with this store.
         runForAllGridsInStore(storeName, function(storeGrid) {
            CPBDataGrid.clearGridAndReload(storeGrid);
         });
         
         // hide masks on each grid.
         runForAllGridsInStore(storeName, function(storeGrid){
            CPBDataGrid.hideMask(storeGrid);
         });
      }
   });
};

/**
 * Refresh a single grid.  Does not reload remote store data.
 */
CPBDataGrid.refreshGrid = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   grid.cpb_refresh();
};

/**
 * Refreshes all grids sharing a single store.  Does not reload remote store data.
 */
CPBDataGrid.refreshGridsSharingStore = function(storeName)
{
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.showMask(storeGrid);
   });
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.refreshGrid(storeGrid);
   });
   runForAllGridsInStore(storeName, function(storeGrid) {
      CPBDataGrid.hideMask(storeGrid);
   });
};

CPBDataGrid.addRowActionsColumn = function(gridName, columnDefs, rowActions, lazyActionsUrl)
{
   if (lazyActionsUrl || (rowActions && rowActions.length > 0))
   {
      var renderCellFunction = function(rowData, data, node, options) {
         
         // Note: rowData.rowIndex is typically undefined, but is set for GEG's
         var rowIndex = rowData.rowIndex;
         
         button = new DropDownButton({
            showLabel: false
         });
         
         if (!lazyActionsUrl) {
            button.dropDown = createStandardActionMenu(gridName, button, rowActions, rowIndex, rowData);
         }
         else {
            enableLazyActionMenu(gridName, button, rowActions, rowIndex, rowData, lazyActionsUrl);
         }
         
         node.appendChild(button.domNode);

      };
      
      var column = { field: 'actionMenuColumn', label: ' ', resizable: false, unhidable: true, sortable: false, renderCell: renderCellFunction };
      columnDefs.push(column);
      
   }
};

CPBDataGrid.getRowCount = function(grid)
{
   if (isString(grid))
   {
      grid = CPBDataGrid.getGrid(grid);
   }
   return grid.cpb_totalRowCount;
};

CPBDataGrid.getURL = function(gridName, uiContextID, cacheDataInUIContext)
{
   var action = !cacheDataInUIContext ? 'allRowData' : 'rowData';
   return 'controller/datagrid/' + action + '/' + gridName + '/' + uiContextID + '.json';
};

function isEmpty(obj) {
   for(var prop in obj) {
       if(obj.hasOwnProperty(prop))
           return false;
   }
   return true;
};

function getSelectorType(args)
{
   if (args.selectorType && args.selectorType!=='default') {
      return args.selectorType;
   }
   var gridActions = args.gridActions;
   if (gridActions && gridActions.length>0) {
      for (var i=0; i<gridActions.length; i++) {
         if (gridActions[i].sas_selectionButton) {
            return 'checkbox';
         }
      }
   }
   return 'none';
};

var ctrlEquiv = has("mac") ? "metaKey" : "ctrlKey";

// summary:
//   Creates a DataGrid using the specified parameter map.
// args:
//   Parameters to create the grid.  Recognized parameters are:
// gridId:
//   The id to give the grid.
// gridName:
//   the name to give the grid.
// storeName:
//   The name of the data store.
// selectedKeyField:
//   the "key" field of the data for each row.
// gridDiv:
//   The div to render the grid into.
// columns:
//   Column data for the grid.
// store:
// gridActions:
// readOnly:
// fireOnChangeEvents:
// fireOnSelectEvents:
// refreshListeners:
// pageSize:
// pageSizeOptions:
CPBDataGrid.createGrid = function(args)
{
   var selectionMode = null;
   if (!args.readOnly) {
      var selectorType = getSelectorType(args);
      if (selectorType!=='none') {
         var columnParams = { field: 'checkboxes', selectorType: selectorType, resizable: false, unhidable: true };
         if (selectorType==='checkbox') {
            selectionMode = 'multiple';
         }
         else {
            selectionMode = 'single';
            columnParams.label = ' '; // Workaround for dgrid bug
         }
         var column = selector(columnParams);
         args.columns.splice(0, 0, column);
      }
   }
   if (!selectionMode) {
      selectionMode = 'single';
   }
   
   var addAllPageSize = false;
   if (args.pageSizeOptions && args.pageSizeOptions.length) {
      
      // dgrid has no means to provide labels for its select options, so we take special care for 'all'
      var index = array.indexOf(args.pageSizeOptions, 'all');
      if (index > -1) {
         args.pageSizeOptions.splice(index, 1);
         addAllPageSize = true;
      }
      
      // Make sure the initial page size is in the size choice list
      if (args.pageSize && array.indexOf(args.pageSizeOptions, args.pageSize)==-1) {
         args.pageSizeOptions.push(args.pageSize);
      }
      args.pageSizeOptions.sort(function(a, b) { return a<b ? -1 : (a>b ? 1 : 0) });
      
   }
   
   var grid = new (declare([Grid, selector, Selection, Pagination, ColumnResizer, ColumnHider/*, ColumnReorder*/, DijitRegistry]))(
   {
      id: args.gridId,
      columns: args.columns,
      store: args.store,
      selection: {},
      selectionMode: selectionMode,
      allowSelectAll: true,
      deselectOnRefresh: true,
      loadingMessage: '<div style="display: inline-block" class="dijitIconLoading"></div> ' + AppProperties['table.default.loading.message.txt'],
      noDataMessage: AppProperties['table.default.no.rows.message.txt'],
      visible: args.visible,
      cpb_id: args.gridId,
      cpb_name: args.gridName,
      cpb_storeName: args.storeName,
      cpb_hasOnChangeEvents: args.fireOnChangeEvents,
      cpb_selectedKeyField: args.selectedKeyField,
      _cpb_standby: null,
      rowsPerPage: args.printMode ? 100000000 : args.pageSize,
      firstLastArrows: args.printMode ? false : true,
      pageSizeOptions: args.printMode ? [] : args.pageSizeOptions || [],
      pagingLinks: args.printMode ? 0 : 2,
      
      postCreate: function() 
      {  
         // We do this stuff here in postCreate because if we do it after dgrid creation,
         // it's too late for some stuff like initially-disabled row actions due to maxRows
         var grid = this;
         // Register all refreshListeners
         if (args.refreshListeners && args.refreshListeners.length > 0) {
            array.forEach(args.refreshListeners, function(listener) {
               grid.on('dgrid-refresh-complete', listener.onrefreshListener);
            });
         }
         grid.on('dgrid-error', function(error) {
            // Slow-loading grids may still be loading data on a page redirect.
            // In this case, don't show errors stemming from their canceled Ajax requests.
            if (!cpb.isRedirecting) {
               cpb.consoleLog(error.toString(), 'error');
               grid.cpb_showMessage(AppProperties['table.default.errorLoadingData.message.txt'], 'error');
            }
         });
         // Cache the total row count from the store
         aspect.after(grid, 'renderArray', function(results) {
            if (typeof results.total === 'number')
            {
               grid.cpb_totalRowCount = results.total;
               grid._fixHeight(false);
               grid.resize();
            }
            else
            {
               results.total.then(function(results) {
                  grid.cpb_totalRowCount = results;
                  grid._fixHeight(false);
                  grid.resize();
               });
            }
         }, true);
         this.inherited(arguments);
      },
      
      cpb_getSelectedRowValues: function(colName) {
         colName = colName || this.cpb_selectedKeyField;
         var values = [];
         for (var id in this.selection) {
            // selection value may be "false" if using "select all" checkbox and individual checkboxes
            if (this.selection[id]) {
               var rowData = this.row(id);
               if (rowData.data[colName]!=null) { // Don't push null or undefined, but do push 0 and false
                  values.push(rowData.data[colName]);
               }
            }
         }
         return values;
      },
      
      cpb_getAllRowValues: function(colName) {
         colName = colName || this.cpb_selectedKeyField;
         var values = [];
         this.store.query({}).forEach(function(record) {
            values.push(record[colName]);
         });
         return values;
      },
      
      cpb_refresh: function(flushStore) {
         flushStore = flushStore || false;
         
         if (!grid._started)
         {
            cpb.consoleLog("Grid " + grid.cpb_name + " not yet started; calling startup()...");
            grid.startup();
         }
         else
         {
            cpb.consoleLog("Grid " + grid.cpb_name + " started; refreshing...");
            
            if (flushStore) {
               CPBDataGrid.refreshServerStore(grid.cpb_storeName);
            }
            else {
               RefreshingGridTracker.addRefreshingGrid(grid);
               CPBService.clearGridData(grid.cpb_name, cpb.getFormParams(), 
                  function(isCleared) 
                  {
                     CPBDataGrid.clearGridAndReload(grid.cpb_name);
                     if (grid.cpb_hasOnChangeEvents)
                     {
                        cpb.fieldValueChanged(grid.cpb_name);
                     }
                  }
               );
            }
         }
      },
      
      cpb_showMessage: function(message, type) {
         CPBDataGrid.showMessage(this, message, type);
      },
      
      _destroyChildrenRecursively: function() {
         var widgets = registry.findWidgets(this.domNode);
         array.forEach(widgets, function(widget) {
            widget.destroyRecursive(false);
         });
      },
      
      _fixHeight: function(fix) {
         var scroller = query('.dgrid-scroller', this.domNode)[0];
         var height = fix ? dojo.position(scroller).h + 'px' : '';
         scroller.style.minHeight = height;
      },
      
      gotoPage: function(page, focusLink) {
         // Don't shrink the grid height while fetching more data from server
         this._fixHeight(true);
         this._destroyChildrenRecursively();
         CPBDataGrid._ignoreSelectEvents = true;
         this.clearSelection(); // Why is this needed?
         var result = this.inherited(arguments);
         CPBDataGrid._ignoreSelectEvents = false;
         if (args.fireOnSelectEvents) { cpb.fieldSelected(this.id, {}); }
         return result;
      },

      _setSort: function(property, descending) {
         if (this._isSorting) { // dgrid issue #635
            return;
         }
         this._isSorting = true;
         // Don't shrink the grid height while fetching more data from server
         this._fixHeight(true);
         this._destroyChildrenRecursively();
         CPBDataGrid._ignoreSelectEvents = true;
         this.inherited(arguments);
         CPBDataGrid._ignoreSelectEvents = false;
         if (args.fireOnSelectEvents) { cpb.fieldSelected(this.id, {}); }
      },

      // Extends ColumnReorder._destroyColumns()
      _destroyColumns: function() {
         this._destroyChildrenRecursively();
         this.inherited(arguments);
      },
      
      // _handleSelect overridden to prevent firign two on-select events when the user clicks from one row
      // to another.  No good way to give advice, etc. for this.
      _handleSelect: function(event, currentTarget){
         // don't run if selection mode is none,
         // or if coming from a dgrid-cellfocusin from a mousedown
         if(this.selectionMode == "none" ||
               (event.type == "dgrid-cellfocusin" && event.parentType == "mousedown") ||
               (event.type == "mouseup" && currentTarget != this._waitForMouseUp)){
            return;
         }
         // Don't select the row if the user is clicking a row action button
         if (!domClass.contains(event.target, 'dgrid-cell')) {//domClass.contains(event.target, 'dijitArrowButtonInner') || domClass.contains(event.target, 'dijitButtonNode')) {
            return;
         }
         this._waitForMouseUp = null;
         this._selectionTriggerEvent = event;
         var ctrlKey = !event.keyCode ? event[ctrlEquiv] : event.ctrlKey;
         if(!event.keyCode || !event.ctrlKey || event.keyCode == 32){
            var mode = this.selectionMode,
               row = currentTarget,
               rowObj = this.row(row),
               lastRow = this._lastSelected;
            
            if(mode == "single"){
               if(lastRow === row){
                  // Allow ctrl to toggle selection, even within single select mode.
                  this.select(row, null, !ctrlKey || !this.isSelected(row));
               }else{
                  // Don't fire on-select events if we're just clearing the selection for a new one
                  this._cpb_skipFiringSelectEvents = true;
                  this.clearSelection();
                  delete this._cpb_skipFiringSelectEvents;
                  this.select(row);
                  this._lastSelected = row;
               }
            }else if(this.selection[rowObj.id] && !event.shiftKey && event.type == "mousedown"){
               // we wait for the mouse up if we are clicking a selected item so that drag n' drop
               // is possible without losing our selection
               this._waitForMouseUp = row;
            }else{
               var value;
               // clear selection first for non-ctrl-clicks in extended mode,
               // as well as for right-clicks on unselected targets
               if((event.button != 2 && mode == "extended" && !ctrlKey) ||
                     (event.button == 2 && !(this.selection[rowObj.id]))){
                  this.clearSelection(rowObj.id, true);
               }
               if(!event.shiftKey){
                  // null == toggle; undefined == true;
                  lastRow = value = ctrlKey ? null : undefined;
               }
               this.select(row, lastRow, value);
      
               if(!lastRow){
                  // update lastRow reference for potential subsequent shift+select
                  // (current row was already selected by earlier logic)
                  this._lastSelected = row;
               }
            }
            if(!event.keyCode && (event.shiftKey || ctrlKey)){
               // prevent selection in firefox
               event.preventDefault();
            }
         }
         this._selectionTriggerEvent = null;
      },
      
      _cpb_fixPaginationLinks: function() {
         // Fix dgrid pagination links
         // NOTE: This is fixed with dgrid 0.3.7, so remove this fix in the next release!
         var paginationLinks = query('a.dgrid-first, a.dgrid-previous, a.dgrid-page-link, a.dgrid-next, a.dgrid-last', this.domNode);
         if (paginationLinks && paginationLinks.length>0) {
            for (var i=0; i<paginationLinks.length; i++) {
               var a = paginationLinks[i];
               // Replace links with href='javascript:;' to href='#' and call e.preventDefault()
               if (!/#$/.test(a.href)) {
                  a.href = '#';
                  a.addEventListener('click', function(e) { e.preventDefault(); });
               }
            }
         }
      },
_updateNavigation: function() {
   this.inherited(arguments);
   this._cpb_fixPaginationLinks();
},

      showMask: function() {
         if (this._cpb_standby) {
            return; // Grid is already masked
         }
         var div = dojo.byId('gridAndToolbarDiv_' + this.cpb_name);
         this._cpb_standby = new Standby({ target: div });
         document.body.appendChild(this._cpb_standby.domNode);
         this._cpb_standby.show();
      },
      
      hideMask: function() {
         if (this._cpb_standby) {
            this._cpb_standby.hide();
            delete this._cpb_standby;
         }
      }
      
   }, args.gridName);
   CPBDataGrid.gridMap[args.gridName] = grid;
   
   if (addAllPageSize) {
      var select = query('.dgrid-page-size', grid.domNode)[0];
      var label = AppProperties['table.default.all.rows.selectChoice.txt'];
      domConstruct.create('option', { innerHTML: label, value: 1000000 }, select);
   }
   if (args.pageSizeOptions && !args.printMode) {
      var select = query('.dgrid-page-size', grid.domNode)[0];
      select.value = grid.rowsPerPage;
   }
   
   grid.on('dgrid-refresh-complete', function(evt) {
      
      // Let grid resize to whatever's appropriate for its data
      //grid._fixHeight(false);
      if (grid._isSorting) {
         delete grid._isSorting;
      }
      grid.resize(); // Column/column header alignment issue on first render
      
   });

   if (args.gridActions && args.gridActions.length>0)
   {
      var toolbarId = "gridToolbar_" + args.gridName;
      var toolbar = new Toolbar({}, toolbarId);
      for (var i = 0; i < args.gridActions.length; i++)
      {
         toolbar.addChild(args.gridActions[i]);
      }
      
      var selectionChangedHandler = function(event) {
         
         // Update any grid actions dependent on a selection
         var selection = event.grid.selection;
         var toolbarId = "gridToolbar_" + args.gridName;
         var toolbar = dijit.byId(toolbarId);
         if (toolbar)
         {
            var selection = !isEmpty(selection);
            var children = toolbar.getChildren();
            for (var i = 0; i < children.length; i++)
            {
               if (children[i].sas_selectionButton)
               {
                  children[i].set('disabled', !selection);
               }
            }
         }
         
      };
      grid.on('dgrid-select', selectionChangedHandler);
      grid.on('dgrid-deselect', selectionChangedHandler);
    
   }
   
   // on-select events may also need to be triggered
   if (args.fireOnSelectEvents)
   {
      _addOnSelectAdvice(grid);
   }

   if (document.body.dir === 'rtl') {
      BaseGrid.fixDgridRtlBugs(grid);
   }
   
   return grid;
};

CPBDataGrid._ignoreSelectEvents = false;

_addOnSelectAdvice = function(grid)
{
   aspect.after(grid, 'select', function(row, toRow, value) {
      if (CPBDataGrid._ignoreSelectEvents) {
         return;
      }
      // See select() method for argument values and their meanings
      if(value === undefined){
         // default to true
         value = true;
      } 
      if (!row.element) {
         row = this.row(row);
      }
      var selected = this.selection[row.id];
      var lastSelectedRowData = selected ? row.data : {};
      cpb.fieldSelected(grid.id, lastSelectedRowData);
   }, true);
   
   // dgrid will call select() for each row individually in a selectAll() call, but
   // we don't want to fire 20+ on-select events.  We only want to fire one.
   aspect.before(grid, 'selectAll', function() {
      CPBDataGrid._ignoreSelectEvents = true;
   });
   aspect.after(grid, 'selectAll', function() {
      CPBDataGrid._ignoreSelectEvents = false;
      // on-select is not technically supported for multi-select grids, but we'll be nice and return an
      // arbitrary selected value
      var selectedKey = null;
      for (var item in this.selection) {
         selectedKey = item;
         break;
      }
      if (selectedKey) {
         cpb.fieldSelected(grid.id, this.row(selectedKey).data);
      }
   });
   
   // dgrid will call deselect() for each row individually in a clearSelection() call, but
   // we don't want to fire 20+ on-select events.  We only want to fire one.
   aspect.before(grid, 'clearSelection', function() {
      CPBDataGrid._ignoreSelectEvents = true;
   });
   aspect.after(grid, 'clearSelection', function() {
      CPBDataGrid._ignoreSelectEvents = false;
      if (grid._cpb_skipFiringSelectEvents) {
         cpb.consoleLog('Not firing select events for grid as a new selection is imminent');
      }
      else {
         cpb.consoleLog('Selection cleared with no imminent new selection, firing on-select');
         cpb.fieldSelected(grid.id, {});
      }
   });
   
};

CPBDataGrid.javaScriptGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         try {
            eval(data.url);
         } catch (err) {
            var message = 'Could not eval JS GridAction: ' + err;
            cpb.consoleLog(message, 'error');
            var grid = CPBDataGrid.getGrid(gridVarName);
            CPBDataGrid.showMessage(grid, message, 'error');
         }
      }
   );
};

CPBDataGrid.inlinePostGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = new URL(data.url);
         var gridAjaxPost = window['urlAjaxPost_' + gridVarName];
         // use QueryString's toUnencodedString() instead of toString(), since the QueryString was already encoded in the Java tier.
         // gridAjaxPost(url.path, url.queryString? url.queryString.toUnencodedString() : null, selectedKeys, callback);
         // toUnencodedString() merely reproduces the string without further encoding
         // toDecodedString() actually decodes the string
         gridAjaxPost(url.path, url.queryString? url.queryString.toDecodedString() : null, selectedKeys, callback);
      }
   );
};

CPBDataGrid.inlineGetGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         var gridAjaxGet = window['urlAjaxGet_' + gridVarName];
         gridAjaxGet(data.url, selectedKeys, callback);
      }
   );
};

CPBDataGrid.newWindowGridActionHandler = function(actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = data.url + (data.url.indexOf("?")==-1 ? "?" : "&") + "keys=" + selectedKeys;
         window.open(url, '_blank');
      }
   );
};

CPBDataGrid.windowGridActionhandler = function(actionId, gridVarName, selectedKeys, callback) {
   CPBService.getGridActionURLData(actionId, cpb.uiContextId, cpb.screenId, cpb.getFormParams(),
      function(data) {
         if (data.validationErrors) {
            showValidationErrors(data.validationErrors);
            return;
         }
         var url = data.url + (data.url.indexOf("?")==-1 ? "?" : "&") + "keys=" + selectedKeys;
         if (cpb.redirectHandler != null) {
            cpb.redirectHandler(url);
         }
         else {
            window.location.href = url;
         }
      }
   );
};

showValidationErrors = function(errors)
{
   require(['messages/Messages'], function(Messages) {
      new Messages({ errors: errors }).showMessages();
   });
};

CPBDataGrid.showMessage = function(grid, message, type)
{
   require(['messages/Messages'], function(Messages) {
      Messages.showMessage(message, type);
   });
};

// private functions

runForAllGridsInStore = function(storeName, process)
{
   for (var gridName in CPBDataGrid.gridMap)
   {
      var storeGrid = CPBDataGrid.getGrid(gridName);
      if (storeGrid.cpb_storeName === storeName)
      {
         process(storeGrid);
      }
   }
};

// summary:
//   Creates a row-level action menu when all grid actions are known at render time.
createStandardActionMenu = function(gridName, parent, rowActions, row, rowData)
{
   var actionMenu = new DropDownMenu({ style: 'display: none;' });
   var menuItems = createRowActionsMenuItems(gridName, rowActions, row, rowData);
   for (var i = 0; i < menuItems.length; i++) 
   {
      actionMenu.addChild(menuItems[i]);
   }
   return actionMenu;
};

// summary:
//   Creates a row-level action menu that lazily fetches (some of) its child menu items when it's first displayed.
enableLazyActionMenu = function(gridName, parent, rowActions, row, rowData, lazyActionsUrl)
{
   var grid = CPBDataGrid.getGrid(gridName);
   var queryString = 'key=' + rowData[grid.cpb_selectedKeyField];
   var questionMark = lazyActionsUrl.indexOf('?');
   if (questionMark>-1) {
      queryString += '&' + lazyActionsUrl.substring(questionMark+1);
      lazyActionsUrl = lazyActionsUrl.substring(0, questionMark);
   }
   
   var args = {
      query: queryString,
      handleAs: 'json',
      sync: true
   };

   parent.loadDropDown = function(callback) {
      xhr(lazyActionsUrl, args).then(function(data) {
         // Add laziliy-fetched menu items first.
         parent.dropDown = CPBMenu.buildMenu(parent, data.menuItemList);
         // Add any user-defined menu items next.
         var userDefinedItems = createRowActionsMenuItems(gridName, rowActions, row, rowData);
         for (var i=0; i<userDefinedItems.length; i++) {
            parent.dropDown.addChild(userDefinedItems[i]);
         }
         if (callback) { callback(); }
      }, function(err) {
         var error = err;
         CPBDataGrid.showMessage(grid, 'An error occurred loading the action menu.', 'error');
      });
      //this.inherited(arguments);
   };
   
};

createRowActionsMenuItems = function(gridName, rowActions, row, rowData)
{
   var items = [];
   
   if (rowActions && rowActions.length) {
      for (var i = 0; i < rowActions.length; i++) 
      {
         var action = rowActions[i];
         items.push(new MenuItem({
            label: action.label,
            disabled: action.disabled,
            iconClass: action.iconClass,
            onClick: function(delegate) {
                        return function()  {
                           // Set some variables useful for JS actions
                           var grid = CPBDataGrid.getGrid(gridName);
                           actionData = {
                              rowIndex: row, // For backwards compatibility, not set anymore with dgrid
                              gridName: gridName,
                              _grid: grid,
                              checkedRowKeys: CPBDataGrid.getSelectedRowKeys(grid),
                              selectedRowKey: rowData[grid.cpb_selectedKeyField],
                              selectedRowData: rowData 
                           };
                           delegate.onClick();
                        };
                     }(action)
         }));
      }
   }
   
   return items;
};

   return CPBDataGrid;
   
});