define([
    'dojox/html/entities',
    './Attachment',
    'dojox/lang/functional',
    '../CommentData',
    './ConfirmDialog',
    'dijit/Dialog',
    '../AttachmentUploader/AttachmentUploader',
    './ReplyForm',
    'dojo/dom-style',
    'dojo/dom-construct',
    'dojo/on',
    'dojo/mouse',
    'dojo/dom-class',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetBase',
    'dijit/_WidgetsInTemplateMixin',
    'dijit/form/Button',
    'dojo/_base/declare',
    'dojo/i18n!./nls/validate',
    'dojo/text!./templates/Comment.html',
    'fraudCommons/util/NLSUtil'
], function(entities, Attachment, functional, CommentData, ConfirmDialog, Dialog, AttachmentUploader, ReplyForm, domStyle, domConstruct, on, mouse, domClass, TemplatedMixin, WidgetBase, WidgetsInTemplateMixin, Button, declare, validate, template, NLSUtil ) {

    return declare([WidgetBase, TemplatedMixin, WidgetsInTemplateMixin], {
    	templateString: template,
    	
    	constructor: function( args ) {
    		   this.readOnly = args.readOnly || false;
            this.author = args.author;
            this.commentId = args.commentId;
            this.objectId = args.objectId;
            this.objectType = args.objectType;
            this.objectName = args.objectName;
            this.parentId = args.parentId;
            this.date = args.date;
            this.editable = (!this.readOnly && args.editable) || false;
            this.deletable = (!this.readOnly && args.deletable) || false;
            this.attachmentsEditable = args.attachmentsEditable || false;
            this.attachments = args.attachments || [];
            this.onComplete = args.onComplete;
            this.truncationLength = args.truncationLength || 500;
    	},
    	
        buildRendering: function() {
        	this.inherited(arguments);
        },
        
        postMixinProperties: function() {
        	this.inherited(arguments);
        },
        
        startup: function() {
        	this.inherited(arguments);
        },
        
        postCreate: function() {
            var that = this;
            var truncationLength = this.truncationLength;
        	   this.inherited(arguments);
            this.authorContainer.innerHTML = this.author;
            this.dateContainer.innerHTML = this.date;
            this.editButton.innerHTML = NLSUtil.resolve('btn_edit', validate);
            this.deleteButton.innerHTML = NLSUtil.resolve('btn_delete', validate);
            
            var isDecoded = function(s) {
               return entities.decode(s) === s;
            };
            
            var decodeHtml = function(s) {
               var txt = document.createElement("textarea");
               txt.innerHTML = s;
               return txt.value;
            };
            
            var fixText = function(s) {
               s = s.replace(/\t/g, "&emsp;");
               var splitString = s.split(/\n/g);
               for(var i = 0; i < splitString.length; i++) {
                  splitString[i] = splitString[i].replace(/ +/g, function(match, offset, string) {
                     if(offset === 0) {
                        return match.replace(/\s/g, '&nbsp;');
                     } else {
                        return match.substring(0, 1) + match.substring(1).replace(/\s/g, '&nbsp;');
                     }
                  });
               }
               return splitString.join("<br />");
            };
            
            if(this.readOnly || (!this.editable && !this.deletable)) {
               if (this.editToolbar && this.editToolbar.domNode){
                  domStyle.set(this.editToolbar.domNode, {'display': 'none'});
               }
            } else {
                if(this.readOnly || (!this.editable && this.editButton)) {
                   domStyle.set(this.editButton, {'display': 'none'});
                }
                if(this.readOnly || (!this.deletable && this.deleteButton)) {
                   domStyle.set(this.deleteButton, {'display': 'none'});
                }
                if(!this.editable || !this.deletable) {
                    this.separator.destroy();
                }
            }

            on(this.editButton, "click", function() {
                that._displayEditToolbar(false);
                var form = new ReplyForm({
                    objectId: that.objectId,
                    objectName: that.objectName,
                    objectType: that.objectType,
                    subject: that.subject
                });

                var attachmentUploader = new AttachmentUploader({
                    commentId: that.commentId,
                    d_form: form,
                    url: 'comments',
                    editForm: true,
                    attachments: that.attachments,
                    dataCallback:function () {
                        return {
                            'commentId': that.commentId,
                            'objectId': that.objectId,
                            'objectType': that.objectType,
                            'objectName': that.objectName,
                            'subject': that.subject,
                            'text' : form.textContainer.get('value')
                        };
                    },
                    cancelCallback: function() {
                        form.destroy();
                        that._displayEditToolbar(true);
                    },
                    onComplete: function() {
                        that.onComplete();
                    }
                });
                
                form.textContainer.set('value', decodeHtml(that.text));
                attachmentUploader.startup();
                on(form.textContainer, "change", function() {
                    attachmentUploader.disableButtons(!form.textContainer.textbox.value.trim().length > 0);
                });
                domConstruct.place(form.domNode, that.attachmentsContainer, "before");
                
            });

            on(this.deleteButton, "click", function() {
                var dialog = new ConfirmDialog({
                    title: NLSUtil.resolve('delete_confirm_title', validate),
                    style: 'width: 350px',
                    message: NLSUtil.resolve(that.parentId == 0 ? 'delete_topic_confirm_message' : 'delete_reply_confirm_message', validate),
                    confirmLabel: NLSUtil.resolve('btn_confirm', validate),
                    cancelLabel: NLSUtil.resolve('btn_cancel', validate),
                    onConfirm: function() {
                        dialog.destroy();
                        CommentData.deleteComment(that.commentId, that.onComplete);
                    },
                    onCancel: function() {
                        dialog.destroy();
                    }
                });
                dialog.show();
            });

            functional.forEach(this.attachments, function(attachment, key) {
                var li = new Attachment({
                    title: attachment.name,
                    href: 'ViewAttachmentServlet?a_id=' + attachment.id
                });
                domConstruct.place(li.domNode, that.attachmentsContainer, "last");
            });
            
            var decodedText = decodeHtml(this.text);
            if(this.text && decodedText.length > truncationLength) {
                var s = decodedText.substring(0, truncationLength);
                var content = fixText(entities.encode(s));
                
                this.textContainer.set('content', content + NLSUtil.resolve('ellipses', validate));

                this.showMoreButton = new Button({
                    label: NLSUtil.resolve('show_more', validate),
                    onClick: function() {
                        that.textContainer.set('content', fixText(that.text));
                        that.showMoreButton.set({ disabled: true });
                        that.showLessButton.set({ disabled: false });
                        domStyle.set(that.showMoreButton.domNode, {'display': 'none'});
                        domStyle.set(that.showLessButton.domNode, {'display': 'block'});
                        that.collapseText.addChild(that.showLessButton);
                    }
                });

                this.showLessButton = new Button({
                    label: NLSUtil.resolve('show_less', validate),
                    onClick: function() {
                        var s = decodedText.substring(0, truncationLength);
                        that.textContainer.set('content', fixText(entities.encode(s)) + NLSUtil.resolve('ellipses', validate));
                        that.showMoreButton.set({ disabled: false });
                        that.showLessButton.set({ disabled: true });
                        domStyle.set(that.showMoreButton.domNode, {'display': 'block'});
                        domStyle.set(that.showLessButton.domNode, {'display': 'none'});
                        that.collapseText.addChild(that.showMoreButton);
                    }
                });

                domClass.add(this.showMoreButton.domNode, "invisibleButton");
                domClass.add(this.showLessButton.domNode, "invisibleButton");
                this.collapseText.addChild(this.showMoreButton);
            } else {
                this.textContainer.set('content', fixText(this.text));
            }
        },

        _displayEditToolbar: function(bool) {
            var that = this;
            if (that.attachmentsContainer) {
               domStyle.set(that.attachmentsContainer, {
                   display: bool ? 'block' : 'none'
               });
            }
            if (that.textContainer) {
               domStyle.set(that.textContainer.domNode, {
                  display: bool ? 'block' : 'none'
              });
            }
            if (that.editToolbar) {
               domStyle.set(that.editToolbar.domNode, {
                  display: bool ? 'block' : 'none'
              });
            }
            if (that.collapseText) {
               domStyle.set(that.collapseText.domNode, {
                  display: bool ? 'block' : 'none'
              });
            }
        },

        expandText: function() {
            this.textContainer.innerHTML = this.text;
        },

        resize: function() {
            this.inherited(arguments);
        },

        truncate: function(numLines) {
        }
        
    });
});
