/***********************************************************************************/
/* Copyright (c) 2012, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_prvw_fincen_sar_subj                                       */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Transform the subject data for report preview                    */
/* PARAMETERS:                                                                     */
/*                rr_config_rk: key of the report to preview (required)            */
/* NOTE:          called by ecmrr_prvw_fincen_sar_subj                             */
/*                all subject related tables should be transformed into rectangular*/
/*                structure.                                                       */
/***********************************************************************************/
%macro ecmrr_prvw_fincen_sar_subj;


%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

/* concatenate email addresses */
%ecmrr_fincen_concat_txt(indsn=zx_subject_email_address
                            ,outdsn=z_subject_email_address
                            ,byvar=rr_rk z_subject_seq_no
                            ,invar=x_email_address_txt
                            ,outvar=zx_email_address_txt
                            ,delimiter=%str(, )
                            ,maxsize=32760
                            ,sortedYN=N
                            ,sortby=rr_rk z_subject_seq_no
                            ,fmtnm=
                           );

/* concatenate alternate names */
%ecmrr_fincen_concat_txt(indsn=zx_subject_alternate_nm
                            ,outdsn=z_subject_alternate_nm
                            ,byvar=rr_rk z_subject_seq_no
                            ,invar=x_alternate_nm
                            ,outvar=zx_alternate_nm
                            ,delimiter=%str(, )
                            ,maxsize=32760
                            ,sortedYN=N
                            ,sortby=rr_rk z_subject_seq_no
                            ,fmtnm=
                           );

/* concatenate url addresses */
%ecmrr_fincen_concat_txt(indsn=zx_subject_url_address
                            ,outdsn=z_subject_url_address
                            ,byvar=rr_rk z_subject_seq_no
                            ,invar=x_url_address_txt
                            ,outvar=zx_url_address_txt
                            ,delimiter=%str(, )
                            ,maxsize=32760
                            ,sortedYN=N
                            ,sortby=rr_rk z_subject_seq_no
                            ,fmtnm=
                           );

/* transpose telephone records into phone columns */
/* max number of phones is 4                      */
data zx_telephone_transposed;
     set zx_subject_telephone_no;
     by z_subject_seq_no;

     array zx_phone_type_cd {4} $1 zx_phone_type_cd_1 - zx_phone_type_cd_4;
     array zx_phone_no {4} $16 zx_phone_no_1 - zx_phone_no_4;
     array zx_phone_ext_no {4} $6 zx_phone_ext_no_1 - zx_phone_ext_no_4;
    
     retain zx_phone_type_cd_1 - zx_phone_type_cd_4
            zx_phone_no_1      - zx_phone_no_4
            zx_phone_ext_no_1  - zx_phone_ext_no_4
            
     ;

     if first.z_subject_seq_no then link ARRAY_CLEAR;
     array_i+1;
     if array_i > 4 then
     do;
         output;
         link ARRAY_CLEAR;
         array_i=1;
     end;
     link ARRAY_ASSIGN;
     if last.z_subject_seq_no then output;
     return;

ARRAY_CLEAR:
    do i=1 to 4;
       zx_phone_type_cd{i}='';
       zx_phone_no{i}='';
       zx_phone_ext_no{i}='';
    end;
    array_i=0;
return;

ARRAY_ASSIGN:

       zx_phone_type_cd{array_i}=x_phone_type_cd;
       zx_phone_no{array_i}=x_phone_no;
       zx_phone_ext_no{array_i}=x_phone_ext_no;


return;

run;

/* create header field for labelling subject sequence number */
%let z_subject_cnt=%ecm_nobs(zx_subject);

proc sql;
     create table z_subject as
     select *,
            'Part I Subject Information '||strip(put(z_subject_seq_no,best.))||
                  " of &z_subject_cnt"
            as z_header_txt length=100
     from zx_subject
     order by z_subject_seq_no
;

/* order address data before merging with subject data */
proc sort data=zx_subject_address force;
     by z_subject_seq_no row_no;
run;

/* prepare subject data for sar_s4 */

data zx_subject_p1
     t_zx_subject_address_more (keep=
                            x_address_txt
                            x_city_nm
                            x_state_cd
                            x_country_cd
                            x_postal_cd
                            x_address_unknown_flg
                            x_city_nm_unknown_flg
                            x_state_unknown_flg
                            x_country_unknown_flg
                            x_postal_unknown_flg
                            z_subject_seq_no
                            z_address_seq_no
                            z_header_txt
                        )
                        
     t_zx_subject_id_more (keep=
                            x_subject_id_type_cd
                            x_subject_id_type_oth_txt
                            x_id_no
                            x_id_authority_state_cd
                            x_id_authority_country_cd
                            x_subject_id_type_oth_txt
                            z_subject_seq_no
                            z_header_txt
                            z_subject_id_seq_no
                        )
                    
                          
;
     merge z_subject
           zx_subject_address (in=in_address rename=(row_no=z_address_seq_no))
           z_subject_alternate_nm
           z_subject_email_address
           z_subject_url_address
           zx_telephone_transposed
           zx_subject_id (in=in_id rename=(row_no=z_subject_id_seq_no)) 
     ;
     by z_subject_seq_no;
     retain first_subject_id_seq_no first_address_seq_no;
     /* keep only the first address of the subject for sar_s4 */
     /* other addresses go to subject_address_more table for sar_s6 */
     if first.z_subject_seq_no then 
     do;
         output zx_subject_p1;
         first_subject_id_seq_no=z_subject_id_seq_no;
         first_address_seq_no=z_address_seq_no;
     end;
     else 
     do;
          if in_address and z_address_seq_no ne first_address_seq_no then 
              output t_zx_subject_address_more;
          if in_id  and z_subject_id_seq_no ne first_subject_id_seq_no then 
              output t_zx_subject_id_more; 
    end;
run;



proc sort data=t_zx_subject_address_more nodupkey;
     by z_subject_seq_no z_address_seq_no;
run;



/* prepare subject_address_more for sar_s6 */

%let repeat=7;
data zx_subject_address_more;
     set t_zx_subject_address_more;
     by z_subject_seq_no;


     array z_address_txt {&repeat} $100 x_address_txt_1 - x_address_txt_&repeat;
     array z_city_nm {&repeat} $50 x_city_nm_1 - x_city_nm_&repeat;
     array z_postal_cd {&repeat} $9 x_postal_cd_1 - x_postal_cd_&repeat;
     array z_state_cd {&repeat} $3 x_state_cd_1 - x_state_cd_&repeat;
     array z_country_cd {&repeat} $2 x_country_cd_1 - x_country_cd_&repeat;
     
     array z_address_unknown_flg {&repeat} $1 x_address_unknown_flg_1 - x_address_unknown_flg_&repeat;
     array z_city_nm_unknown_flg {&repeat} $1 x_city_nm_unknown_flg_1 - x_city_nm_unknown_flg_&repeat;
     array z_postal_unknown_flg {&repeat} $1 x_postal_unknown_flg_1 - x_postal_unknown_flg_&repeat;
     array z_state_unknown_flg {&repeat} $1 x_state_unknown_flg_1 - x_state_unknown_flg_&repeat;
     array z_country_unknown_flg {&repeat} $1 x_country_unknown_flg_1 - x_country_unknown_flg_&repeat;


     retain
            x_address_txt_1 - x_address_txt_&repeat
            x_city_nm_1 - x_city_nm_&repeat
            x_postal_cd_1 - x_postal_cd_&repeat
            x_state_cd_1 - x_state_cd_&repeat
            x_country_cd_1 - x_country_cd_&repeat
            x_address_unknown_flg_1 - x_address_unknown_flg_&repeat
            x_city_nm_unknown_flg_1 - x_city_nm_unknown_flg_&repeat
            x_postal_unknown_flg_1 - x_postal_unknown_flg_&repeat
            x_state_unknown_flg_1 - x_state_unknown_flg_&repeat
            x_country_unknown_flg_1 - x_country_unknown_flg_&repeat


     ;

     if first.z_subject_seq_no then link ARRAY_CLEAR;
     array_i+1;
     if array_i > &repeat then
     do;
         output;
         link ARRAY_CLEAR;
         array_i=1;
     end;
     link ARRAY_ASSIGN;
     if last.z_subject_seq_no then output;
     return;

ARRAY_CLEAR:
    do i=1 to &repeat;

       z_address_txt{i}='';
       z_city_nm{i}='';
       z_postal_cd{i}='';
       z_state_cd{i}='';
       z_country_cd{i}='';       
    
       z_address_unknown_flg{i}='';
       z_city_nm_unknown_flg{i}='';
       z_postal_unknown_flg{i}='';
       z_state_unknown_flg{i}='';
       z_country_unknown_flg{i}='';

    end;
    array_i=0;
return;

ARRAY_ASSIGN:


       z_address_txt{array_i}=x_address_txt;
       z_city_nm{array_i}=x_city_nm;
       z_postal_cd{array_i}=x_postal_cd;
       z_state_cd{array_i}=x_state_cd;
       z_country_cd{array_i}=x_country_cd;
       z_address_unknown_flg{array_i}=x_address_unknown_flg;
       z_city_nm_unknown_flg{array_i}=x_city_nm_unknown_flg;
       z_postal_unknown_flg{array_i}=x_postal_unknown_flg;
       z_state_unknown_flg{array_i}=x_state_unknown_flg;
       z_country_unknown_flg{array_i}=x_country_unknown_flg;



return;

run;

/* prepare subject_id_more data for sar_s7 */

proc sort data=t_zx_subject_id_more nodupkey;
     by z_subject_seq_no z_subject_id_seq_no;
run;

%let repeat=3;
data zx_subject_id_more;
     set t_zx_subject_id_more;
     by z_subject_seq_no;


     array z_subject_id_type_cd {&repeat} $1 x_subject_id_type_cd_1 - x_subject_id_type_cd_&repeat;
     array z_subject_id_type_oth_txt {&repeat} $50 x_subject_id_type_oth_txt_1 - x_subject_id_type_oth_txt_&repeat;
     array z_id_no {&repeat} $24 x_id_no_1 - x_id_no_&repeat;
     array z_id_authority_state_cd {&repeat} $2 x_id_authority_state_cd_1 - x_id_authority_state_cd_&repeat;
     array z_id_authority_country_cd {&repeat} $3 x_id_authority_country_cd_1 - x_id_authority_country_cd_&repeat;
     
     retain
            x_subject_id_type_cd_1 - x_subject_id_type_cd_&repeat
            x_id_no_1 - x_id_no_&repeat
            x_id_authority_state_cd_1 - x_id_authority_state_cd_&repeat
            x_id_authority_country_cd_1 - x_id_authority_country_cd_&repeat
            x_subject_id_type_oth_txt_1 - x_subject_id_type_oth_txt_&repeat

     ;

     if first.z_subject_seq_no then link ARRAY_CLEAR;
     array_i+1;
     if array_i > &repeat then
     do;
         output;
         link ARRAY_CLEAR;
         array_i=1;
     end;
     link ARRAY_ASSIGN;
     if last.z_subject_seq_no then output;
     return;

ARRAY_CLEAR:
    do i=1 to &repeat;

       z_subject_id_type_cd{i}='';
       z_id_no{i}='';
       z_id_authority_state_cd{i}='';
       z_id_authority_country_cd{i}='';
       z_subject_id_type_oth_txt{i}='';

    end;
    array_i=0;
return;

ARRAY_ASSIGN:


       z_subject_id_type_cd{array_i}=x_subject_id_type_cd;
       z_subject_id_type_oth_txt{array_i}=x_subject_id_type_oth_txt;
       z_id_no{array_i}=x_id_no;
       z_id_authority_state_cd{array_i}=x_id_authority_state_cd;
       z_id_authority_country_cd{array_i}=x_id_authority_country_cd;
       



return;

run;


proc sort data=zx_subject_relation_inst force;
     by z_subject_seq_no x_subject_relation_inst_rk;
run;

proc sort data=zx_subject_inst_acct nodupkey force;
      by z_subject_seq_no  x_acct_inst_rk  x_acct_closed_flg x_acct_id ;
run;

/* concatenate accounts */
 
data zx_subject_inst_acct_transposed;
     set zx_subject_inst_acct;
     array z_acct_id{2} $ 4096 z_acct_id_1- z_acct_id_2 ;
     array z_acct_closed_flg{2} $ 1 z_acct_closed_flg_1 - z_acct_closed_flg_2 ;
     retain z_acct_id_1 z_acct_id_2 z_acct_closed_flg_1 z_acct_closed_flg_2;
     retain array_i 0;
     by  z_subject_seq_no  x_acct_inst_rk  x_acct_closed_flg;
     if first.x_acct_inst_rk then link CLEAR_ARRAY;

     if first.x_acct_closed_flg then 
     do; 
         array_i+1; 
         z_acct_id{array_i}=x_acct_id;   
         z_acct_closed_flg{array_i}=x_acct_closed_flg; 
     end;
     else    
        z_acct_id{array_i}=strip(z_acct_id{array_i})||' '||strip(x_acct_id);
      
     if last.x_acct_inst_rk then output;
     return;

CLEAR_ARRAY:

do array_i=1 to 2;
   z_acct_id{array_i}='';
   z_acct_closed_flg{array_i}='';
end;
array_i=0;

run;
 
/* prepare subject and institution relation data for sar_s5 */


data zx_subject_p2
     t_zx_subject_relation_inst_more (keep= 
                            zx_subject_relation_oth_txt
                            x_subject_relation_status_cd
                            x_subject_relation_status_dt
                            zx_subject_relation                             
                            z_subject_seq_no
                            x_inst_tin_no
                            z_header_txt
                            x_subject_relation_inst_rk
                        )
      zx_subject_inst_acct_more (keep= 
                            x_acct_inst_id
                            z_acct_id_1 z_acct_id_2
                            z_acct_closed_flg_1 z_acct_closed_flg_2
                            x_acct_inst_non_us_flg
                            z_header_txt
                            z_subject_seq_no
                            x_acct_inst_rk
                           
                        )
; 
     retain first_subject_relation_inst_rk first_acct_inst_rk;
     merge z_subject (keep=z_subject_seq_no z_header_txt  x_no_acct_flg)
           zx_subject_relation_inst (in=in_relation )
           zx_subject_inst_acct_transposed (in=in_acct )

           
     ;
     by z_subject_seq_no;
     if first.z_subject_seq_no then 
     do;
          first_subject_relation_inst_rk=x_subject_relation_inst_rk;
          first_acct_inst_rk=x_acct_inst_rk;
          output zx_subject_p2;
     end;
     else 
     do;
        if in_relation and x_subject_relation_inst_rk ne first_subject_relation_inst_rk then 
           output t_zx_subject_relation_inst_more;
        if in_acct and x_acct_inst_rk ne first_acct_inst_rk then 
           output zx_subject_inst_acct_more;
    end;
run;

proc sort data= t_zx_subject_relation_inst_more nodupkey;
     by z_subject_seq_no x_subject_relation_inst_rk;
run;


proc sort data=  zx_subject_inst_acct_more nodupkey;
     by z_subject_seq_no x_acct_inst_rk;
run;

/* prepare subject_relation_inst_more for additional subject/institution relation data (sar_s9) */
%let repeat=7;
data zx_subject_relation_inst_more;
     set t_zx_subject_relation_inst_more;
     by z_subject_seq_no;


     array z_subject_relation_status_cd {&repeat} $1 x_subject_relation_status_cd_1 - x_subject_relation_status_cd_&repeat;
     array z_subject_relation_status_dt {&repeat} 8 x_subject_relation_status_dt_1 - x_subject_relation_status_dt_&repeat;
     array z_subject_relation {&repeat} $12 zx_subject_relation_1 - zx_subject_relation_&repeat;
     array z_subject_relation_oth_txt {&repeat} $50 zx_subject_relation_oth_txt_1 - zx_subject_relation_oth_txt_&repeat;
     array z_inst_tin_no {&repeat} $25 x_inst_tin_no_1 - x_inst_tin_no_&repeat;
    

     retain
            x_subject_relation_status_cd_1 - x_subject_relation_status_cd_&repeat
            x_subject_relation_status_dt_1 - x_subject_relation_status_dt_&repeat
            zx_subject_relation_1 - zx_subject_relation_&repeat
            zx_subject_relation_oth_txt_1 - zx_subject_relation_oth_txt_&repeat
            x_inst_tin_no_1 - x_inst_tin_no_&repeat

     ;

     if first.z_subject_seq_no then link ARRAY_CLEAR;
     array_i+1;
     if array_i > &repeat then
     do;
         output;
         link ARRAY_CLEAR;
         array_i=1;
     end;
     link ARRAY_ASSIGN;
     if last.z_subject_seq_no then output;
     return;

ARRAY_CLEAR:
    do i=1 to &repeat;

       z_subject_relation_status_cd{i}='';
       z_subject_relation_status_dt{i}=.;
       z_subject_relation{i}='';
       z_subject_relation_oth_txt{i}='';
       z_inst_tin_no{i}='';
        
    end;
    array_i=0;
return;

ARRAY_ASSIGN:

       z_subject_relation_status_cd{array_i}=x_subject_relation_status_cd;
       z_subject_relation_status_dt{array_i}=x_subject_relation_status_dt;
       z_subject_relation{array_i}=zx_subject_relation;
       z_subject_relation_oth_txt{array_i}=zx_subject_relation_oth_txt;
       z_inst_tin_no{array_i}=x_inst_tin_no;
      


return;

run;

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;
MEXIT:
%mend;
